set END; #Menge der Endprodukte
set BAU; #Menge der Baugruppen
set TL; # Menge der Teile
set PROD:= END union BAU union TL;

param bigM;
param erloesEnd{END} >=0; # Erloes fuer die Endprodukte
param erloesBau{BAU} >=0; # Erloes fuer die verkauften Baugruppen
param kostBau{BAU} >=0; # Kosten zur Erstellung der Baugruppen
param kostTl{TL} >=0; # Kosten der Teile
param maxAbsatz{PROD} default bigM; # Absatzobergrenze: nur fuer Endprodukte gegeben 
param minAbsatz{PROD} default 0; # Absatzuntergrenze: nur fuer Endprodukte gegeben 
param minAbsatzVerkauf{BAU} default 20; # Absatzuntergrenze fuer die verkauften Baugruppen
param zeit{BAU} >=0; # Zeit fuer den Bau der Baugruppen
param kapZeit >=0; # maximale Zeitkapazitaet
param anfLagTl{TL} >=0; # Anfangslagerbestand der Teile
param anfLagBau{BAU} >=0; # Anfangslagerbestand der Baugruppen
param stueckliste{TL union BAU,BAU union END} >=0; # Stueckliste

param grmenge;

param sondpreis >=0; # Aufgabe b: Sonderpreis fuer Teile
param kostrabatt >=0; # Aufgabe c: Rabatt fuer Teile

var xmenge{p in PROD} >= minAbsatz[p], <= maxAbsatz[p]; # Anzahl der produzierten Endprodukte

var xbauverkauf{b in BAU} >= minAbsatzVerkauf[b]; # Menge der Verkauften Baugruppen

var xsondTl >=0, <=200; # Aufgabe b: Menge der gekauften Teile von E1 zum Sonderpreis

var xmenrabatt >=0; # Aufgabe c: Menge der Teile E1 zum Rabatt

var xrabatt binary; # Aufgabe c: 1, Rabatt wird fuer E1 gewaehrt sonst 0

# Zielfunktion: maximiere Gewinn -> Erloes Endprodukte + Erloes Baugruppe - Kosten Baugruppe - Kosten Teile
maximize Gewinn: sum{e in END} erloesEnd[e]*xmenge[e] + sum{b in BAU} (erloesBau[b]*xbauverkauf[b]-kostBau[b]*xmenge[b]) - sum{t in TL} kostTl[t]*xmenge[t]
                 - sondpreis*xsondTl - kostrabatt*xmenrabatt;

# Bilanzgleichung fuer alle Baugruppen: Verbrauch fuer Baugruppen und Endprodukte + Verkauf Baugruppe - Anfangslagerbestand <= Menge der Baugruppe
s.t. BilanzBau {b in BAU}: sum{e in END union BAU} stueckliste[b,e]* xmenge[e] + xbauverkauf[b] - anfLagBau[b] <= xmenge[b];

# Bilanzgleichung der Teile: Verbrauch fuer Baugruppe - Anfangslagerbestand <= Menge der Teile
# Durch die Verallgemeinerung der Teile zur Menge TL, darf nur bei t=E1 xsondTl+xmenrabatt auf xprod addiert werden 
s.t. BilanzTL {t in TL}: sum{b in BAU} stueckliste[t,b]*xmenge[b] - anfLagTl[t]  <= xmenge[t]  + (if (t in {'E1'})then xsondTl + xmenrabatt );

# Zeitkapazitaet: 
s.t. ZeitKapazitaet: sum{b in BAU} zeit[b]*xmenge[b] <= kapZeit;

# Aufgabe c
# Wenn mehr als 500 ME von den Teilen benoetigt werden gibt es einen Rabatt => xrabatt[t] = 1
# => durch die Maximierung des Erloeses ist xprod[t] maximal 500
s.t. MengenrabattTL1: xmenge['E1'] >= grmenge*xrabatt;
# Wenn kein Rabatt gewaehrt wird, dann ist xmenrabatt gleich Null
s.t. MengenrabattTL2: xmenrabatt <= bigM*xrabatt;  
